@@ -10,7 +10,7 @@ use warnings;
 
 use Carp;
 
-our $VERSION = '0.21_002';
+our $VERSION = '0.21_003';
 
 require Exporter;
 our @EXPORT_OK;
@@ -30,8 +30,7 @@ foreach my $impl (qw( Core XS Emul )) {
 
 =head1 NAME
 
-C<Socket::GetAddrInfo> - RFC 2553's C<getaddrinfo> and C<getnameinfo>
-functions.
+C<Socket::GetAddrInfo> - address-family independent name resolving functions
 
 =head1 SYNOPSIS
 
@@ -39,21 +38,23 @@ functions.
  use Socket::GetAddrInfo qw( getaddrinfo getnameinfo );
  use IO::Socket;
 
- my $sock;
-
  my %hints = ( socktype => SOCK_STREAM );
  my ( $err, @res ) = getaddrinfo( "www.google.com", "www", \%hints );
 
  die "Cannot resolve name - $err" if $err;
 
- while( my $ai = shift @res ) {
+ my $sock;
 
-    $sock = IO::Socket->new();
-    $sock->socket( $ai->{family}, $ai->{socktype}, $ai->{protocol} ) or
-       undef $sock, next;
+ foreach my $ai ( @res ) {
+    my $candidate = IO::Socket->new();
 
-    $sock->connect( $ai->{addr} ) or undef $sock, next;
+    $candidate->socket( $ai->{family}, $ai->{socktype}, $ai->{protocol} )
+       or next;
 
+    $candidate->connect( $ai->{addr} )
+       or next;
+
+    $sock = $candidate;
     last;
  }
 
@@ -143,6 +144,10 @@ sub import
 
 =head2 ( $err, @res ) = getaddrinfo( $host, $service, $hints )
 
+C<getaddrinfo> turns human-readable text strings (containing hostnames,
+numeric addresses, service names, or port numbers) into sets of binary values
+containing socket-level representations of these addresses.
+
 When given both host and service, this function attempts to resolve the host
 name to a set of network addresses, and the service name into a protocol and
 port number, and then returns a list of address structures suitable to
@@ -166,7 +171,31 @@ fields:
 
 =item flags => INT
 
-A bitfield containing C<AI_*> constants
+A bitfield containing C<AI_*> constants. At least the following flags will be
+available:
+
+=over 2
+
+=item * C<AI_PASSIVE>
+
+Indicates that this resolution is for a local C<bind()> for a passive (i.e.
+listening) socket, rather than an active (i.e. connecting) socket.
+
+=item * C<AI_CANONNAME>
+
+Indicates that the caller wishes the canonical hostname (C<canonname>) field
+of the result to be filled in.
+
+=item * C<AI_NUMERICHOST>
+
+Indicates that the caller will pass a numeric address, rather than a hostname,
+and that C<getaddrinfo> must not perform a resolve operation on this name.
+This flag will prevent a possibly-slow network lookup operation, and instead
+return an error, if a hostname is passed.
+
+=back
+
+Other flags may be provided by the OS.
 
 =item family => INT
 
@@ -219,15 +248,59 @@ address.
 
 =head2 ( $err, $host, $service ) = getnameinfo( $addr, $flags, $xflags )
 
-This function attempts to resolve the given socket address into a pair of host
-and service names.
+C<getnameinfo> turns a binary socket address into a pair of human-readable
+strings, containing the host name, numeric address, service name, or port
+number.
 
 The optional C<$flags> parameter is a bitfield containing C<NI_*> constants.
+At least the following flags will be available:
+
+=over 2
+
+=item * C<NI_NUMERICHOST>
+
+Requests that a human-readable string representation of the numeric address is
+returned directly, rather than performing a name resolve operation that may
+convert it into a hostname.
+
+=item * C<NI_NUMERICSERV>
+
+Requests that the port number be returned directly as a number representation
+rather than performing a name resolve operation that may convert it into a
+service name.
+
+=item * C<NI_NAMEREQD>
+
+If a name resolve operation fails to provide a name, then this flag will cause
+C<getnameinfo> to indicate an error, rather than returning the numeric
+representation as a human-readable string.
+
+=item * C<NI_DGRAM>
+
+Indicates that the socket address relates to a C<SOCK_DGRAM> socket, for the
+services whose name differs between C<TCP> and C<UDP> protocols.
+
+=back
+
+Other flags may be provided by the OS.
 
 The optional C<$xflags> parameter is a bitfield containing C<NIx_*> constants.
-The two constants are C<NIx_NOHOST> and C<NIx_NOSERV>. These are a Perl-level
-extension of the API to request looking up only the service name or host name
-respectively, when only one is required.
+These are a Perl-level extension to the API, to indicate extra information.
+
+=over 2
+
+=item * C<NIx_NOHOST>
+
+Indicates that the caller is not interested in the hostname of the result, so
+it does not have to be converted; C<undef> will be returned as the hostname.
+
+=item * C<NIx_NOSERV>
+
+Indicates that the caller is not interested in the service name of the result,
+so it does not have to be converted; C<undef> will be returned as the service
+name.
+
+=back
 
 Errors are indicated by the C<$err> value returned; which will be non-zero in
 numeric context, and contain a string error message as a string. The value can
@@ -237,6 +310,88 @@ which provides functions that throw exceptions on errors.
 
 =cut
 
+=head1 EXAMPLES
+
+=head2 Lookup for C<connect>
+
+The C<getaddrinfo> function converts a hostname and a service name into a list
+of structures, each containing a potential way to C<connect()> to the named
+service on the named host.
+
+ my %hints = ( socktype => SOCK_STREAM );
+ my ( $err, @res ) = getaddrinfo( $hostname, $servicename, \%hints );
+ die "Cannot getaddrinfo - $err" if $err;
+
+ my $sock;
+
+ foreach my $ai ( @res ) {
+    my $candidate = IO::Socket->new();
+
+    $candidate->socket( $ai->{family}, $ai->{socktype}, $ai->{protocol} )
+       or next;
+
+    $candidate->connect( $ai->{addr} )
+       or next;
+
+    $sock = $candidate;
+    last;
+ }
+
+Because a list of potential candidates is returned, the C<while> loop tries
+each in turn until it it finds one that succeeds both the C<socket()> and
+C<connect()> calls.
+
+This function performs the work of the legacy functions C<gethostbyname>,
+C<getservbyname>, C<inet_aton> and C<pack_sockaddr_in>.
+
+=head2 Making a human-readable string out of an address
+
+The C<getnameinfo> function converts a socket address, such as returned by
+C<getsockname> or C<getpeername>, into a pair of human-readable strings
+representing the address and service name.
+
+ my ( $err, $hostname, $servicename ) = getnameinfo( $socket->peername );
+ die "Cannot getnameinfo - $err" if $err;
+
+ print "The peer is connected from $hostname\n";
+
+Since in this example only the hostname was used, the redundant conversion of
+the port number into a service name may be omitted by passing the
+C<NIx_NOSERV> flag.
+
+ my ( $err, $hostname ) = getnameinfo( $socket->peername, 0, NIx_NOSERV );
+
+This function performs the work of the legacy functions C<unpack_sockaddr_in>,
+C<inet_ntoa>, C<gethostbyaddr> and C<getservbyport>.
+
+=head2 Resolving hostnames into IP addresses
+
+To turn a hostname into a human-readable plain IP address use C<getaddrinfo>
+to turn the hostname into a list of socket structures, then C<getnameinfo> on
+each one to make it a readable IP address again.
+
+ my ( $err, @res ) = getaddrinfo( $hostname, "", { socktype => SOCK_RAW } );
+ die "Cannot getaddrinfo - $err" if $err;
+
+ while( my $ai = shift @res ) {
+    my ( $err, $ipaddr ) = getnameinfo( $ai->{addr}, NI_NUMERICHOST, NIx_NOSERV );
+    die "Cannot getnameinfo - $err" if $err;
+
+    print "$ipaddr\n";
+ }
+
+The C<socktype> hint to C<getaddrinfo> filters the results to only include one
+socket type and protocol. Without this most OSes return three combinations,
+for C<SOCK_STREAM>, C<SOCK_DGRAM> and C<SOCK_RAW>, resulting in triplicate
+output of addresses. The C<NI_NUMERICHOST> flag to C<getnameinfo> causes it to
+return a string-formatted plain IP address, rather than reverse resolving it
+back into a hostname.
+
+This combination performs the work of the legacy functions C<gethostbyname>
+and C<inet_ntoa>.
+
+=cut
+
 =head1 BUILDING WITHOUT XS CODE
 
 In some environments it may be preferred not to build the XS implementation,
